/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004-2005 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Font functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//-------------------------------------------------------------------- CONSTANTS

#define MAX_FONTS			256

//-------------------------------------------------------------------- VARIABLES

// Local

static PFONT pFonts[MAX_FONTS];			// Font cache -- font index zero is not used
static int nInstalledFonts = 0;
static int nLastFont = 0;				// Zero means no font

//---------------------------------------------------------- FUNCTION PROTOTYPES

//----------------------------------------------------------- EXPORTED FUNCTIONS

/* Sets the font of control pwbobj. Returns the index of the created font or
  zero if failed. */

int wbCreateFont(LPCTSTR pszName, int nHeight, COLORREF color, DWORD dwFlags)
{
	HFONT hFont;
	HDC hdc;

	if(nInstalledFonts >= MAX_FONTS - 1)
		return 0;

	// Convert points to pixels

    hdc = GetDC(NULL);
    nHeight = -MulDiv(nHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72);
    ReleaseDC(NULL, hdc);

	// Create font with attributes

	hFont = CreateFont(nHeight,
	    0, 0, 0,
		dwFlags & FTA_BOLD ? FW_BOLD : FW_NORMAL,
		dwFlags & FTA_ITALIC,
		dwFlags & FTA_UNDERLINE,
		0, DEFAULT_CHARSET, 0,0,0,0,
		pszName);

	// Store font in cache

	if(hFont) {
		nInstalledFonts++;
		nLastFont = nInstalledFonts;
		pFonts[nLastFont] = wbMalloc(sizeof(FONT));
		if(!pFonts[nInstalledFonts])
			return 0;
		pFonts[nLastFont]->pszName = wbMalloc(strlen(pszName) + 1);
		if(!pFonts[nLastFont]->pszName)
			return 0;
		strcpy(pFonts[nLastFont]->pszName, pszName);
		pFonts[nLastFont]->nHeight = nHeight;
		pFonts[nLastFont]->color = color;
		pFonts[nLastFont]->dwFlags = dwFlags;
		pFonts[nLastFont]->hFont = hFont;
	} else
		return 0;

	return nLastFont;
}

/* Sets the font of control pwbobj.

  TODO: change text color
*/

BOOL wbSetControlFont(PWBOBJ pwbo, int nFont, BOOL bRedraw)
{
	if(!wbIsWBObj(pwbo, TRUE))					// Is it a valid control?
		return FALSE;

	if(!IsWindow(pwbo->handle))
		return FALSE;

	if(nFont > nInstalledFonts)
		return FALSE;

	SendMessage(pwbo->handle, WM_SETFONT, (WPARAM)pFonts[nFont]->hFont,
	  MAKELPARAM(bRedraw, 0));

	if(nFont <= 0)
		nFont = nLastFont;
	else
		nLastFont = nFont;
/*
	// Doesn't work, should intercept WM_CTLCOLOR messages
{
	HDC hdc;
    hdc =  GetWindowDC(pwbo->handle);
	SetTextColor(hdc, pFonts[nFont]->color);
	printf("%d %d %08X\n", hdc, pwbo->handle, pFonts[nFont]->color);
    ReleaseDC(pwbo->handle, hdc);
}
*/
	return TRUE;
}

/* Returns the font indexed by nFont. If nFont is <= 0, returns the last font used. */

PFONT wbGetFont(int nFont)
{
	if(nFont > nInstalledFonts)
		return NULL;

	if(nFont <= 0)
		nFont = nLastFont;
	else
		nLastFont = nFont;

	return pFonts[nFont];
}

// TODO: Not tested yet; not exported to PHP yet

BOOL wbDestroyFonts(void)
{
	int i;

	for(i = 0; i < nInstalledFonts; i++) {
		if(!DeleteObject(pFonts[i]->hFont))
			return FALSE;
		if(!wbFree(pFonts[i]->pszName))
			return FALSE;
		if(!wbFree(pFonts[i]))
			return FALSE;
	}
	nInstalledFonts = 0;
	return TRUE;
}

//------------------------------------------------------------------ END OF FILE
